#!/usr/bin/env python3

'''
              _     _ _ _        __ _           _
 _ __ ___  __| | __| (_) |_ ___ / _(_)_ __   __| | ___ _ __
| '__/ _ \/ _` |/ _` | | __/ __| |_| | '_ \ / _` |/ _ \ '__|
| | |  __/ (_| | (_| | | |_\__ \  _| | | | | (_| |  __/ |
|_|  \___|\__,_|\__,_|_|\__|___/_| |_|_| |_|\__,_|\___|_|


github.com/fitzy1293/redditsfinder


TO DO:
    - Make the module useful when imported
'''

#import urllib.request
import requests
import json
import time
import os,sys
#from urllib.error import HTTPError
from pathlib import Path

from rich.table import Table,Column
from rich.console import Console

from .redditsfinder_utils import *

CONSOLE = Console()

#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def getPosts(user, postType, printIt=True): # Pushshift API requests in chunks of 100
    console = Console()
    if printIt:
        console.print(f'[bold blue underline]{postType[0].upper()}{postType[1:]} request log:')

    postSetMaxLen = 100 # Max num of posts in each pushshift request, seems to be 100 right now or it breaks.

    return pushshift(None, postType, postSetMaxLen, user=user, log=printIt)
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def printTotals(dataForTable, maxRecentPosts=50, printIt=True): #Prints table after the pushshift log.

    header = (
                'line #\n',
                'subreddit\n',
                'karma\n',
                'comment\ncount',
                'comment\nkarma',
                'submission\ncount',
                'submission\nkarma'
    )

    table = Table(show_header=True,header_style="bold blue")

    for colTitle in header:
        table.add_column(colTitle, justify='left')


    for i, subredditDictObject in enumerate(dataForTable):
        subTitle = subredditDictObject.get('sub')
        subKarma = subredditDictObject.get('karma')
        commentInfo, submissionInfo  = subredditDictObject.get('submission,comment') #(comment data, submission data)


        table.add_row(
                        f'[bold red]{str(i+1)}[/bold red]',
                        f'[bold magenta underline]{subTitle}[/bold magenta underline]',
                        f'[bold magenta underline]{subKarma}[/bold magenta underline]',
                        f'[bold cyan underline]{commentInfo[0]} [/bold cyan underline]',
                        f'[bold cyan]{commentInfo[1]}[/bold cyan]',
                        f'[bold green underline]{submissionInfo[0]} [/bold green underline]',
                        f'[bold green]{submissionInfo[1]}[/bold green]'
        )

    CONSOLE.print(table, justify='center')

#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────

def run(cliArgs, user='', printIt=True, **kwargs):
    start = time.time()
    console = Console()

    if kwargs:
        cliArgs = ['--' + i for i in kwargs.keys()]

    if 'user' in kwargs.keys():
        user = kwargs['user']

    if printIt:
        console.print(f'[bold blue]Gathering and formatting data from pushshift for {user}.\n')

    if  '-pics' in cliArgs or '--pics' in cliArgs:
        keyType = {'submission': ('url', 'created_utc',)}
        images = imageUrls(user, [i for i in getPosts(user, 'submission', printIt=printIt)])

        images             = set(open(os.path.join(userDir,'images.txt')).read().splitlines())
        imageStatus        = '\n'.join([f'\t[bold green]{i+1} [cyan]{image}' for i, image in enumerate(images)])
        imageSubmissionLog = f'[bold blue underline]\nImages submitted by {user}:[/bold blue underline]\n{imageStatus}'
        if printIt:
            console.print(imageSubmissionLog)

        if '-d' in cliArgs or '--download' in cliArgs:
            imagesdl(images, userDir)
    else:
        allPosts = {'comments': [i for i in getPosts(user, 'comment', printIt=printIt)],
                    'submissions': [i for i in getPosts(user, 'submission', printIt=printIt)]}
        numericInfoBySub = countPosts(allPosts)

        if printIt:
            print()
            printTotals(numericInfoBySub)
            print()
        if '--write' or '-w' in cliArgs:
            userDir = Path.cwd() / 'users' / user
            userDir.mkdir(parents=True, exist_ok=True)
            writeFiles(allPosts, numericInfoBySub, user, userDir)

            if printIt:
                fnamesStr = '\n\t' + '\n\t'.join([i for i in os.listdir(userDir)])
                console.print(f'\n[bold cyan underline]{userDir}{fnamesStr}')

    if printIt:
        console.print(f'[bold blue]Run time - {round(time.time() - start, 1)} (s)[/bold blue]\n[bold cyan]{"-"*50}[/bold cyan]')

#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────

def main():  # System arguments

    if len(sys.argv) == 1:
        print('''Running redditsfinder

        Test it on yourself to make sure it works.
            redditsfinder someusername

        Basic usage
            redditsfinder username
            redditsfinder [options] username_0 username_1 username_2 ...

        With an input file
            -f or --file.
            redditsfinder [options] -f line_separated_text_file.txt

        Examples
            - just print the summary table to stdout
                $ redditsfinder someusername

            - save data locally and print the summary table to stdout
                $ redditsfinder --write someusername

            - just save data locally without printing
                $ redditsfinder --write --quiet someusername

            - download pictures
                $ redditsfinder --pics someusername

        Optional args
            --pics returns URLs of image uploads
            -pics -d or --pics --download downloads them
            -quiet or -q turns off printing''' )


    else:
        redditsfinderArgs = sys.argv[1:]
        optionalArgs = {'--write', '-w', '-pics', '--pics', '-d', '--download', '-q', '--quiet', '-f', '--file'}
        enteredOptionalArgs = set([i for i in redditsfinderArgs if i in optionalArgs])

        if '-f' in enteredOptionalArgs:
            if sys.argv[-1] in os.path.exists():
                with open(sys.argv[-1]) as f:
                    usernames = f.read().splitlines()
            else:
                sys.exit(f'error: {sys.argv[-1]} not found')
        else:
            usernames = [i for i in redditsfinderArgs if i not in optionalArgs]

        usersCt = len(usernames)

        if not '-q' in enteredOptionalArgs and not '--quiet' in enteredOptionalArgs:
            printIt = True
        else:
            printIt=False

        if usersCt > 1 and printIt:
            print('\nnote: this is not done in parallel, an account like automoderator with millions of posts will be blocking\n')

        for user in usernames:
            run(enteredOptionalArgs, user=user, printIt=printIt)

if __name__ == '__main__':
    main()



import time
import requests
import traceback
import os,sys
import imghdr
from datetime import datetime

from zipfile import ZipFile
import json
from pprint import pprint
from pathlib import Path
from rich.table import Table,Column
from rich.console import Console

#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def cleanupPostData(postData, postType): #Thanks /u/kwelzel for helping me refactor!
    interestingPostData = {}
    submissionAndCommentKeys = ('id', 'created_utc', 'subreddit', 'score', 'link_id', 'parent_id')

    for i in submissionAndCommentKeys:
        if not i in postData.keys():
            interestingPostData[i] = None

        else:
            interestingPostData[i] = postData[i]

    timestamp = int(postData['created_utc'])
    interestingPostData['datetime'] = str(datetime.utcfromtimestamp(timestamp).strftime('%a %b %d %Y, %I:%M %p UTC'))

    if postType == 'comment':
        if 'permalink' in postData.keys():
            interestingPostData['permalink'] = f'https://www.reddit.com{postData["permalink"]}'
        else:
            interestingPostData['permalink'] = None
        if 'body' in postData.keys():
            interestingPostData['body'] = textPostWords(postData['body'])
        else:
            interestingPostData['body'] = None

    if postType == 'submission':
        if 'url' in postData.keys():
            interestingPostData['url'] = postData["url"]
        else:
            interestingPostData['url'] = None
        if 'full_link' in postData.keys():
            interestingPostData['full_link'] = postData["full_link"]
        else:
            interestingPostData['full_link'] = None
        if 'selftext' in postData.keys():
            interestingPostData['selftext'] = textPostWords(postData['selftext'])
        else:
            interestingPostData['selftext'] = None

    return interestingPostData
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def pushshift(lim, postType, postSetMaxLen, user='', log=False): #Pushshift requests
    console = Console()
    before = int(round(time.time()))

    if postType == 'comment':
        highlight = '[cyan]'
    else:
        highlight = '[magenta]'

    ct = 0

    while True:

        if lim is not None:
            if ct >= lim:
                break

        apiUrl = 'https://api.pushshift.io/reddit/search/'
        url = f'{apiUrl}{postType}/?author={user}&size={postSetMaxLen}&before={before}'

        response = requests.get(apiUrl + postType, params={'author': user, 'size': postSetMaxLen, 'before': before})
        data = response.json()['data']
        if not data:
            break

        for postData in data:
            before = postData['created_utc']
            yield postData

        ct += len(data)

        if log:
            console.print(f'[bold green]\t{ct} {highlight}{url}')

        if len(data) < postSetMaxLen:
            break

        time.sleep(.75) # rate limiting, pushshift says too bad if you go to quick
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def comments(lim=None, user='', log=False):
    postSetMaxLen = 100
    comment = [i for i in pushshift(lim, 'comment', postSetMaxLen, user=user, log=log)]
    return comment[:lim]
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def submissions(lim=None, user='', log=False):
    postSetMaxLen = 100
    submission = [i for i in pushshift(lim, 'submission', postSetMaxLen, user=user, log=log)]
    return submission[:lim]
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def countPosts(allPosts):  # Count and order most posted subs.

    '''
    Formats data for the table.
    '''
    postSubredditKarmaStrings = []


    commentSubList = [] # To turn into a set
    for i, comment in enumerate(allPosts.get('comments')):
        urlFromIds = f'reddit.com/comments/{comment.get("link_id")[3:]}//{comment.get("id")}'
        sub = comment.get("subreddit")#.lower()
        karma = comment.get("score")
        subKarmaStr = f'{sub}-c-{karma}|{urlFromIds}'

        commentSubList.append(sub)
        postSubredditKarmaStrings.append(subKarmaStr)

    submissionSubList = [] # To turn into a set
    for submission in allPosts.get('submissions'):
        urlFromIds = f'reddit.com/{comment.get("id")}'
        sub = submission.get("subreddit")#.lower()
        karma = submission.get("score")
        subKarmaStr = f'{sub}-s-{karma}|{urlFromIds}'

        submissionSubList.append(sub)
        postSubredditKarmaStrings.append(subKarmaStr)

    postSubredditSet = set(commentSubList) | set(submissionSubList)


    returnList = []
    for uniqSub in sorted(postSubredditSet):
        commentsKarmaTotal = 0
        submissionsKarmaTotal = 0
        postTypeStartIndex = len(uniqSub) + 1
        countPostsList = []
        for post in postSubredditKarmaStrings:
            if post.startswith(f'{uniqSub}-'):

                typePipeKarmaStr = post[postTypeStartIndex:post.find('|')]
                countablePostTypeChar = typePipeKarmaStr[0]

                if countablePostTypeChar == 'c':
                    commentsKarmaTotal += int(typePipeKarmaStr[2:])
                elif countablePostTypeChar == 's':
                    submissionsKarmaTotal+= int(typePipeKarmaStr[2:])



                countPostsList.append(countablePostTypeChar)

        totalKarma    = commentsKarmaTotal + submissionsKarmaTotal
        commentsCt    = countPostsList.count('c')
        submissionsCt = countPostsList.count('s')
        info          = ( (str(commentsCt), str(commentsKarmaTotal)), (str(submissionsCt), str(submissionsKarmaTotal)) )

        returnList.append({'sub': uniqSub, 'karma': totalKarma, 'submission,comment': info})


    sortedByKarmaThenName = sorted(returnList, key = lambda i: (i['karma'], i['sub']), reverse=True)

    return sortedByKarmaThenName
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def writeFiles(allPosts, subredditInfo, user, userDir):
    if len(allPosts) != 0:
        jPath = os.path.join(userDir, 'posts.json')
        with open(jPath, 'w+', newline='\n') as jsonFile:
            json.dump(allPosts, jsonFile, indent=4)

        tPath = os.path.join(userDir, 'count.json')
        with open(tPath, 'w+') as subredditsFile:
            json.dump(subredditInfo, subredditsFile, indent=4)
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def imageUrls(user, submissions):
    urls = [v for i in submissions  for k,v in i.items() if k == 'url']
    imageUrls = []
    for url in urls:
        if url.endswith(('.png', '.PNG', '.jpg', '.JPG', '.gif', '.GIF')):
            imageUrls.append(url)
            continue

        if '://reddit.com' in url:
            continue
        if 'imgur.com' in url:
            if '/a/' in url:
                imageUrls.append(url + '/zip')
            else:
                imageUrls.append(url + '.jpg')

    imageUrlsStr = '\n'.join(imageUrls)
    userDir = Path.cwd() / 'users' / user
    images = os.path.join(str(userDir), 'images.txt')
    with open(images, 'w+') as f:
        f.write(imageUrlsStr+ '\n')
    return imageUrlsStr
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def imagesdl(images, userDir): #Needs to deal with all the nonsense involved with image formatting
    console = Console()
    for i, url in enumerate(images):
        try:
            response = requests.get(url, stream=True)

            if url.endswith(('.png', '.PNG', '.jpg', '.JPG', '.gif', '.GIF')):
                urlType = 'image'
                fileType = f'{url.split(".")[-1]}'
                imagePath = os.path.join(userDir, f'{str(i+1)}.{fileType}')

            else:
                urlType = 'zip'
                fileType = 'urlType'
                imagePath = os.path.join(userDir, f'{i+1}.zip')

            with open(imagePath, 'wb+') as f:
                f.write(response.content)

            dlLog = f'\t{os.path.split(imagePath)[-1]} - Downloaded: {url}'
            console.print(f'[green]{dlLog}')

            try:
                bytes = open(imagePath,'rb').read().decode()[1:9]
                if type(bytes) == str and  bytes[0:2] == 'PK': #A zip file
                    continue

                else:
                    console.print(f'[bold red]Invalid image link - removing {os.path.split(imagePath)[-1]}{" " * 4}')
                    os.remove(imagePath)
                    continue
            except UnicodeDecodeError as e:
                pass

            changedFlag = False
            imghdrExtension = imghdr.what(imagePath)
            if str(imghdrExtension) == 'None':
                imghdrExtension = 'jpeg'
                changedFlag = True

            if urlType == 'image' and not changedFlag:
                newFname = f'{i+1}.{imghdrExtension}'

                newFpath = os.path.join(userDir, newFname)
                os.rename(imagePath, newFpath)

            try:
                with ZipFile(imagePath, 'r') as zipObj:
                    listOfiles = zipObj.namelist()

            except Exception as e:
                if urlType == 'image':
                    continue

                newFname = f'{i+1}.{imghdrExtension}'

                newFpath = os.path.join(userDir, newFname)
                os.rename(imagePath, newFpath)

                logSplit = dlLog.split(' ')
                logSplit[1] = f'{i+1}.{imghdrExtension}'

                changeExtension = str(i+1) + '.zip => ' + newFname

                spaces = len(logSplit[0] + logSplit[1]) - len(changeExtension) + 4
                console.print(f'[bold blue]{changeExtension}{" " * spaces}{e}')

        except Exception as e:
            console.print(f'[bold red]{i} {url} unexpected exception: {traceback.format_exc()}')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['redditsfinder']

package_data = \
{'': ['*']}

install_requires = \
['requests', 'rich']

entry_points = \
{'console_scripts': ['redditsfinder = redditsfinder:main']}

setup_kwargs = {
    'name': 'redditsfinder',
    'version': '2.0.0',
    'description': "Archive a reddit user's post history. Formatted overview of a profile, JSON containing every post, and picture downloads.",
    'long_description': '# redditsfinder - reddit user info\n**`pip3 install redditsfinder`**\n\n**A program to get reddit user post data.**\n\n```Running redditsfinder\n\nTest it on a user to make sure it works.\n    redditsfinder someusername\n\nBasic usage\n    redditsfinder username\n    redditsfinder [options] username_0 username_1 username_2 ...\n\nWith an input file\n    -f or --file.\n    redditsfinder [options] -f line_separated_text_file.txt\n\nExamples\n    - just print the summary table to stdout\n        $ redditsfinder someusername\n\n    - save data locally and print the summary table to stdout\n        $ redditsfinder --write someusername\n\n    - just save data locally without printing\n        $ redditsfinder --write --quiet someusername\n\n    - download pictures\n        $ redditsfinder --pics someusername\n\nOptional args\n    --pics returns URLs of image uploads\n    -pics -d or --pics --download downloads them\n    -quiet or -q turns off printing\n```\n',
    'author': 'fitzy1293',
    'author_email': 'berkshiremind@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Fitzy1293/redditsfinder',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
